# This script creates a sound sieve. All windows which pass 
# throughh the sieve are played. Features are extracted with
# 'extract'. This returns the same identifier as openaudio. Features
# are references with constants such as SFM, FLUX, CENTROID, RMS, PITCH,
# and so on. 'min' and 'max' give the minimum and maximum of the given
# feature for the given collection of windows.

# In order to get this to work, you will have to map the midi messages
# sent by the midi device to are using to the values in the 'sliders' array
# We used a keyboard or a fader box which sent control change messages.


BEGIN { collection = extract("perry.au");
	features[0] = SFM;
	features[1] = SFM;
	features[2] = FLUX;
	features[3] = FLUX;
	features[4] = RMS;
	for(i = 0; i < 5; i++)
	{
		mins[i] = min(collection, features[i]); 
		maxs[i] = max(collection, features[i]);
		print mins[i] " " maxs[i];
	}
	maxs[4] = .15;
	for(i = 0; i < 5; i++)
	{
		sliders[i] = mins[i];
		grain[i] = (maxs[i] - mins[i]) / 128.0;
	}
	sliders[1] = maxs[1];
	sliders[3] = maxs[3];
	volume = .5;
	cur = 0;
	seglen = 0;
	stretch = 1;
	time = 1;
	win = getwindow(collection, LAST);
}

# change this for your midi device.
/ControlChange/ {
	j = MDATA1;
 	if(j<5)	sliders[j] = MDATA2*grain[j] + mins[j]; 
	else
	{
	 if(j == 5) seteffect(TSHIFT, MDATA2/64.0);
	 if(j == 6) seteffect(HOPSZ, MDATA2*100+1);
	 if(j == 7) { volume = MDATA2/32.0; }
	}
}

function is(win, feature, tog, val) {
	feat = getfeature(win, feature);
	if((tog == 1 && feat > val) || (tog == 0 && feat < val)) return 1;
	return 0;
}

/ONTICK/ {
	startwin = win;	
	win = getwindow(collection, NEXT);
	while(!(is(win, SFM, 1, sliders[0]) && 
      		is(win, SFM, 0, sliders[1]) && 
     		is(win, FLUX, 1, sliders[2]) && 
      		is(win, FLUX, 0, sliders[3]) &&
		is(win, RMS, 1, sliders[4]) &&
		is(win, RMS, 0, maxs[4])) && win != startwin) 
	{ 
		win = getwindow(collection, NEXT);
	}
	if(win != startwin) play(win, volume);
}
	 



