#include "Database.h"
#include "MoCollection.h"
#include "MoWindow.h"
#include "MoThing.h"

#include <sys/stat.h>



#define DEFAULT_SIZE 0x100000


Database *Database::instance = 0;

const char *extractor_array[] = {
	"SFFT",
	"MONOPITCH",
	"SFM",
	"ACCENTS",
	"" };
const char *features_array[] = {
	"ROLLOFF",
	"RMS",
	"CENTROID",
	"FLUX",
	"PITCH",
	"PITCHINESS",
	"SFM",
	"AC1",
	"AC2",
	"AC3",
	"AC4",
	"" };
const char *midi_array[] = {
	"MTYPE",
	"MCHAN",
	"MDATA1",
	"MDATA2",
	"MDTIME",
	"" };
const char *descriptor_array[] = {
	"NULL",
	"LIVE",
	"NEXT",
	"PREVIOUS",
	"FIRST",
	"LAST",
	"" };
const char *effect_array[] = {
	"SRATE",
	"DSRATE",
	"PSHIFT",
	"DPSHIFT",
	"TSHIFT",
	"DTSHIFT",
	"HOPSZ",
	"DHOPSZ",
	"PAN",
	"DPAN",
	"SOLA",
	"" };

vector <const char *> extractor_vars;
vector <const char *> feature_vars;
vector <const char *> descriptor_vars;
vector <const char *> midi_vars;
vector <const char *> effect_vars;
vector <System *> extractors;

void
Database::init(const string &dir, int winsize, int hopsize)
{
	base = dir;
	base += "/mofeatures/";
	struct stat stbuf;
	if(stat(base.c_str(), &stbuf)) mkdir(base.c_str(), 0777);

	m_things.reserve(DEFAULT_SIZE);
	m_things.push_back(new MoThing());
	m_collections.push_back(new MoCollection());
	m_windows.push_back(new MoWindow());
	m_windows.push_back(new MoWindow());
	
	winSize = winsize;
	hopSize = hopsize;
}

unsigned int
Database::addFile(MoFile *mof)
{
	MoWindow *window;
	int winSize = mof->src->winSize(), hopSize = mof->src->hopSize(), length = mof->src->iterations();
	for(int i = 0; i < length; i++)
	{
		window = new MoWindow();
		window->file = mof;
		window->file_index = i;
		window->sample_index = i*hopSize;
		window->length = winSize;
		addWindow(window);
		mof->windows.push_back(window);
	}
	cout << "Database::addFile " << m_things.size() << " things registered" << endl;
	addCollection(mof);
	return m_collections.size();
}

unsigned int
Database::addCollection(MoCollection *collection)
{
	collection->global_id = m_things.size();
	collection->collection_id = m_collections.size();
	m_things.push_back(collection);
	m_collections.push_back(collection);
	return m_collections.size();
}

unsigned int
Database::addWindow(MoWindow *window)
{
	window->global_id = m_things.size();
	window->window_id = m_windows.size();
	m_things.push_back(window);
	m_windows.push_back(window);
}

MoCollection *
Database::getCollection(unsigned int index)
{
	if(index >= m_collections.size()) return NULL; 
	MoCollection *collection = m_collections[index];
	if(collection->type != MO_FILE && collection->type != MO_COLLECTION) return NULL;
	return collection;
}

MoFile *
Database::getFile(const string &file)
{
	for(int i = 0; i < m_collections.size(); i++)
	{
		MoThing *mothing = m_collections[i];
		if(mothing->type == MO_FILE)
		{
			MoFile *mofile = (MoFile *)mothing;
			if(mofile->filename->fullname() == file) return mofile;
		}
	}
	return NULL;
}


MoFile *
Database::getFile(int collectid)
{
	if(collectid >= m_collections.size()) return NULL;
	MoThing *mothing = m_collections[collectid];
	if(mothing->type == MO_COLLECTION) return (MoFile *)mothing;
	return NULL;
}


MoWindow *
Database::getWindow(unsigned int index)
{
	if(index >= m_things.size()) return NULL;
	MoThing *thing = m_things[index];
	if(thing->type != MO_WINDOW) return NULL;
	MoWindow *mow = (MoWindow *)thing;
	return (MoWindow *)thing;
}


string
Database::baseFileName(MoFile *mofile)
{
	string name = base;
	name += mofile->filename->name();
	return name;
}

