#include "MoCollection.h"
#include "MoWindow.h"
#include "Util.h"
#include "System.h"
#include "Extractor.h"
#include "FeatExtractor.h"
#include "Database.h"

extern const char *extractor_array[];
extern const char *features_array[];
extern const char *midi_array[];
extern const char *descriptor_array[];
extern const char *effect_array[];

extern vector <const char *> extractor_vars;
extern vector <const char *> feature_vars;
extern vector <const char *> descriptor_vars;
extern vector <const char *> midi_vars;
extern vector <const char *> effect_vars;
extern vector <System *> extractors;


Database *database = Database::Instance();

MoCollection::MoCollection()
{
	type = MO_COLLECTION;
	global_id = collection_id = position = 0;
}


MoCollection::MoCollection(MoCollection *col)
{
	type = MO_COLLECTION;
	global_id = collection_id = position = 0;
	for(int i = 0; i < col->windows.size(); i++)
		windows.push_back(col->windows[i]);
	database->addCollection(this);
}

int
MoCollection::copy(unsigned int moid)
{
	MoCollection *ocol = NULL, *ncol = NULL;
	if((ocol = MoCollection::get(moid)) == NULL) return -1;
	ncol = new MoCollection(ocol);
	return ncol->collection_id;
}

int
MoCollection::addWindow(MoWindow *window)
{
	windows.push_back(window); return 1;
}


int
MoCollection::addWindow(MoWindow *window, MoWindow *place)
{
	int pos = 0;
	while(pos < windows.size() && place->window_id != windows[pos]->window_id) pos++;
	if(pos == windows.size()) return 0;
	windows.insert(windows.begin()+pos, window);
	if(position >= pos) position++;
	if(position == windows.size()) position = 0;
	return 1;
}
		
		
int
MoCollection::deleteWindow(MoWindow *window)
{
	int pos = 0;
	while(pos < windows.size() && window->window_id != windows[pos]->window_id) pos++; 
	if(pos == windows.size()) return 0;
	windows.erase(windows.begin() + pos);
	if(position > pos) position--;
	if(position == windows.size()) position = 0;
	return window->global_id;
}


MoCollection *
MoCollection::get(int col)
{
	return database->getCollection(col);
}



MoFile::MoFile() : MoCollection()
{
	type = MO_FILE;
	global_id = collection_id = 0;
	src = NULL;
	filename = NULL;
	matrix = NULL;
}
	


Util util;

MoFile::MoFile(const string &file, int winSize, int hopSize) : MoCollection()
{
	src = util.sfopen(file.c_str(), MRS_SF_READ);
	src->initWindow(winSize, hopSize, 0, 0);
	filename = new FileName(file);
	database->addFile(this);
}


MoFile *
MoFile::get(const string &name)
{
  MoFile *mof = database->getFile(name);
	if(mof != NULL) return mof;
	return new MoFile(name, database->winSize, database->hopSize);
}

MoFile *
MoFile::get(int fileid)
{
  MoFile *mof = database->getFile(fileid);
	if(mof != NULL) return mof;
	return NULL;
}

int
MoFile::extract()
{
	matrix = new FeatMatrix(src->iterations(), 7);	
	for(int i = 0, curcol = 0; i < NFEATURES; i++)
	{
		//string name = data->baseFileName(mofile);

		FeatMatrix *feats = new FeatMatrix();
		string name = "./featurefiles/";
		name += filename->name();
		name += extractor_vars[i];

		string fstring = name + ".mff";
		struct stat stbuf;
		if(stat(fstring.c_str(), &stbuf))
		{
			System *sys_ = extractors[i];
			Extractor* extr = new FeatExtractor(src, sys_); 
			feats->create(src->iterations(), sys_->outSize());
			extr->extract(*feats);
			cout << "MoCollection::extract Extracted " << extractor_vars[i] << endl;
			feats->write(name.c_str());
			delete extr;
		}
		else
		{
			cout << "MoCollection::extract Reading " << extractor_vars[i] << endl;
			feats->read(name.c_str());
		}
		fvec column(src->iterations());
		for(int j = 0; j < feats->cols(); j++, curcol++)
		{
			column = feats->col(j);
			matrix->col(curcol, column);
		}
		delete feats;
	}
	cout << "MoCollection::extract Extraction for " << filename->name() << " complete" << endl << endl;
	return 1;
}

