/*
 *  MohAwk.cpp
 *  mosievius
 *
 *  Created by Ari Lazier on Sun Mar 30 2003.
 *  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
 *
 */
#include "midiio.h"
#include "MohAwk.h"
#include "MoCollection.h"
#include "MoWindow.h"
#include "Database.h"

#include "awk.h"

#include "defs.h"
#include "System.h"
#include "MSignal.h"
#include "Spectral1.h"
#include "FeatMatrix.h"
#include "Extractor.h"
#include "FeatExtractor.h"
#include "MSignal.h"
#include "Pitch.h"
#include "MFCC.h"
#include "DFCC.h"
#include "FileName.h"
#include "SFM.h"
#include "Accents.h"
#include <iostream>
#include <vector>
#include <pthread.h>

#define MIDI_MESSAGES 1024

Database *data = Database::Instance();
MohAwk *MohAwk::instance = 0;

void *pollmidi(void *m);
void setupMessageTable();
char *midiMessages[MIDI_MESSAGES];

void *processor();
pthread_mutex_t pmutex;
pthread_cond_t pcv;


extern const char *extractor_array[];
extern const char *features_array[];
extern const char *midi_array[];
extern const char *descriptor_array[];
extern const char *effect_array[];

extern vector <const char *> extractor_vars;
extern vector <const char *> feature_vars;
extern vector <const char *> descriptor_vars;
extern vector <const char *> midi_vars;
extern vector <const char *> effect_vars;
extern vector <System *> extractors;

void
MohAwk::init(const string &home, int wsize, int hsize, int tsize)
{
		winSize = wsize;
		hopSize = hsize;
		tickSize = tsize;
		inWin.create(winSize);
		liveWin.create(winSize);

		int i;
		for(i = 0; features_array[i] != ""; i++) feature_vars.push_back(features_array[i]);
		for(i = 0; extractor_array[i] != ""; i++) extractor_vars.push_back(extractor_array[i]);
		for(i = 0; midi_array[i] != ""; i++) midi_vars.push_back(midi_array[i]);
		for(i = 0; descriptor_array[i] != ""; i++) descriptor_vars.push_back(descriptor_array[i]);
		for(i = 0; effect_array[i] != ""; i++) effect_vars.push_back(effect_array[i]);
	
		int j, size = extractor_vars.size() + feature_vars.size() + midi_vars.size() + 
									descriptor_vars.size() + effect_vars.size();
		MohAwkVars 	= new (double *)[size];
		MohAwkCells = new (Cell *)[size];
		for(i = 0, j = 0; i < feature_vars.size(); i++, j++) {
			MohAwkCells[j] = setsymtab(feature_vars[i], "", i, NUM|CON, symtab);
			MohAwkVars[j] = &(MohAwkCells[j])->fval;
		}
		for(i = 0; i < extractor_vars.size(); i++, j++) {
			MohAwkCells[j] = setsymtab(extractor_vars[i], "", i, NUM|CON, symtab);
			MohAwkVars[j] = &(MohAwkCells[j])->fval;
		}
		for(i = 0; i < midi_vars.size(); i++, j++) {
			MohAwkCells[j] = setsymtab(midi_vars[i], "", i, NUM, symtab);
			MohAwkVars[j] = &(MohAwkCells[j])->fval;
		}
		for(i = 0; i < descriptor_vars.size(); i++, j++) {
			MohAwkCells[j] = setsymtab(descriptor_vars[i], "", i, NUM|CON, symtab);
			MohAwkVars[j] = &(MohAwkCells[j])->fval;
		}
		for(i = 0; i < effect_vars.size(); i++, j++) {
			MohAwkCells[j] = setsymtab(effect_vars[i], "", i, NUM|CON, symtab);
			MohAwkVars[j] = &(MohAwkCells[j])->fval;
		}
		cout << "Creating Database..." << endl;		
		data->init(home, winSize, hopSize);
		cout << "Initializing Feature Extractors..." << endl;
		extractorsInit();
		cout << "Starting MIDI..." << endl;
		midiInit();
		cout << "Forking processing thread..." << endl;
		processInit();
		cout << "Creating default ONTICK stream..." << endl << endl;
		current = new Stream("ONTICK", winSize, hopSize, tickSize);
		streams.push_back(current);
}

void
MohAwk::record()
{}

STATUS
MohAwk::play(WINDOW window, FLOAT gain)
{
	if(MoWindow::getWindow(abs(window), inWin) == 0) return 0;
	inWin *= gain;
	current->addwin(window, inWin);
	return 1;
}


STATUS
MohAwk::setstream(STREAM str)
{
	int i = 0;
	while(i < streams.size() && streams[i]->name != str) i++;
	if(i != streams.size())
	{
		current = streams[i];
		return 0;
	}
	current = new Stream(str, winSize, hopSize, tickSize);
	streams.push_back(current);
	return 0;
}


STATUS
MohAwk::seteffect(EFFECT effect, FLOAT value)
{
	return current->effects(effect, value);
}


COLLECTION
MohAwk::openaudio(SOUNDFILE file)
{
	COLLECTION c = (MoFile::get(file))->collection_id;
	return c;
}

COLLECTION
MohAwk::extract(COLLECTION collectid)
{
	MoFile *mofile = MoFile::get(collectid);
	return mofile->extract();
}

COLLECTION
MohAwk::extract(SOUNDFILE name)
{
	int collect = openaudio(name);
	return extract(collect);
}


COLLECTION
MohAwk::newcollection(COLLECTION col)
{
	return MoCollection::copy(col);
}

int
MohAwk::collectionsize(COLLECTION col)
{
	MoCollection *collection = MoCollection::get(col);
	if(collection == NULL) return 0;
	else return collection->windows.size();
}

WINDOW
MohAwk::getwindow(COLLECTION collect, DESCRIPTOR descriptor)
{
	MoCollection *collection = MoCollection::get(collect);
	if(collection == NULL) return 0;
	vector<MoWindow *> &wins = collection->windows;

	if(wins.size() == 0) return 0;
	if(descriptor == symlookup("NEXT")) if((++collection->position) == wins.size()) collection->position = 0;
	if(descriptor == symlookup("FIRST")) collection->position = 0;
	if(descriptor == symlookup("LAST")) collection->position = wins.size()-1;
	if(descriptor == symlookup("PREVIOUS"))
		if(collection->position == 0) collection->position = wins.size()-1;
		else collection->position--;
	return wins[collection->position]->global_id;
}


STATUS
MohAwk::addwindow(COLLECTION collection, WINDOW window, WINDOW place)
{
	MoCollection *col = MoCollection::get(collection);
	MoWindow *win = MoWindow::getId(abs(window)), *pwin = MoWindow::getId(abs(place));
	if(col == NULL || win == NULL) return -1;
	if(place == END) return col->addWindow(win);
	if(pwin == NULL) return -1;
	return col->addWindow(win, pwin);
}


int
MohAwk::deletewindow(COLLECTION collection, WINDOW window)
{
	MoCollection *col = MoCollection::get(collection);
	MoWindow *win = MoWindow::getId(abs(window));
	if(col == NULL) return -1;
	if(win == NULL) return -2;
	return col->deleteWindow(win);
}



float
MohAwk::getfeature(WINDOW window, FEATURE feature)
{
	int i, j, size;
	if(window == symlookup("NULL")) return 0;
	if(window == symlookup("LIVE"))		//	features for live input
	{
		if(!isextracted) for(i = 0, size = 0; i < NFEATURES; i++)
		{
			fvec feats(extractors[i]->outSize());
			extractors[i]->process(liveWin, feats);
			for(j = 0; j < extractors[i]->outSize(); j++, size++) extracted(size) = feats(j);
		}
		isextracted = true;
		return extracted(feature);
	}
	MoWindow *win = MoWindow::getId(abs(window));
	if(win == NULL) return -1;
	FeatMatrix *feats = win->file->matrix;
	if(feats == NULL) return -1;
	return (*feats)(win->file_index, feature);
}


float
MohAwk::min(COLLECTION collect, FEATURE feature)
{
	MoCollection *collection;
	if((collection = MoCollection::get(collect)) == NULL) return -1;
	MoWindow *win = collection->windows[0];
	FeatMatrix *feats;
	feats = win->file->matrix;
	float m = (*feats)(win->file_index, feature);
	for(int i = 0; i < collection->windows.size(); i++)
	{
		win = collection->windows[i];	
		if((*feats)(win->file_index, feature) < m) m = (*feats)(win->file_index, feature);
	}
	return m;
}


float
MohAwk::max(COLLECTION collect, FEATURE feature)
{
	MoCollection *collection;
	if((collection = MoCollection::get(collect)) == NULL) return -1;
	MoWindow *win = collection->windows[0];
	FeatMatrix *feats = win->file->matrix;
	float m = (*feats)(win->file_index, feature);
	for(int i = 0; i < collection->windows.size(); i++)
	{
		win = collection->windows[i];
		if((*feats)(win->file_index, feature) > m) m = (*feats)(win->file_index, feature);
	}
	return m;
}




struct less_mag : public binary_function<MoWindow *, MoWindow *, bool> {
	int sort_feature;
	bool operator()(MoWindow * a, MoWindow * b) {
		return (*a->file->matrix)(a->file_index, sort_feature) < (*b->file->matrix)(b->file_index, sort_feature);
	}
	less_mag(int feature) {
		sort_feature = feature;
	}
};

int
MohAwk::sort(COLLECTION collection, FEATURE feature)
{
	MoCollection *col = NULL;
	if((col = MoCollection::get(collection)) == NULL) return -1;
	::sort(col->windows.begin(), col->windows.end(), less_mag(feature));
	return collection;
}



int
MohAwk::symlookup(const char *str)
{
	return (int)(lookup(str, symtab)->fval);
}

void
MohAwk::tick(int size, fvec &inbuf)
{
	int i;
	isextracted = false;
	for(i = 0; i < hopSize; i++)
	{
		liveWin(i) = liveWin(i+hopSize);
		liveWin(i+hopSize) = inbuf(i);
	}
	tickSize = size;
	pthread_mutex_lock(&pmutex);
	mutex->lock();
	for(i = 0; i < streams.size(); i++) streams[i]->tick(size);
	mutex->unlock();
	pthread_cond_signal(&pcv);
	pthread_mutex_unlock(&pmutex);
}



// initialize extractors for both initial and real time feature extraction

void
MohAwk::extractorsInit()
{
	int i, size, extractor;
	for(i = 0, size = 0; i < extractor_vars.size(); size += extractors.back()->outSize(), i++)
	{
		if(extractor_vars[i] == "SFFT") extractors.push_back(new Spectral1(winSize, 0));
		else if(extractor_vars[i] == "DFCC") extractors.push_back(new DFCC(winSize, 40, 20));
		else if(extractor_vars[i] == "MFCC") extractors.push_back(new MFCC(winSize, 0));
		else if(extractor_vars[i] == "MONOPITCH") extractors.push_back(new Pitch(winSize));
		else if(extractor_vars[i] == "SFM") extractors.push_back(new SFM(winSize));
		else if(extractor_vars[i] == "ACCENTS") extractors.push_back(new Accents(winSize, winSize/2, 4));
		else cerr << "no extractor for " << extractor_vars[i] << endl;
	}
	extracted.create(size);
	return;
}


// procees thread functions - the process thread prepares buffers for each stream between ticks

void
MohAwk::processInit()
{
	processThread = new Thread();
	processThread->start((THREAD_FUNCTION)&processor, (void *)this);
	pthread_mutex_init(&pmutex, NULL);
	pthread_cond_init(&pcv, NULL);
}


void *processor()
{
	MohAwk *mohawk = MohAwk::Instance();
	while(1)
	{
		pthread_mutex_lock(&pmutex);
		pthread_cond_wait(&pcv, &pmutex);

		Stream *temp = mohawk->current;
		for(int i = 0; i < mohawk->streams.size(); i++)
		{
			mohawk->current = mohawk->streams[i];
			mohawk->mutex->lock();
			mohawk->current->process();
			mohawk->mutex->unlock();
			if(mohawk->current != mohawk->streams[i]) temp = mohawk->current;
		}
		mohawk->current = temp;
		pthread_mutex_unlock(&pmutex);
	}
}


// midi thread functions

void
MohAwk::midiInit()
{
	mutex = new Mutex();
	midiactive = true;
	//try { midi = new MidiIn; }
	//catch(StkError &exception) { midiactive = false; }
	midi = new MidiIn;
	midi->open(0);
	if(midiactive == true)
	{
		setupMessageTable();
		midiThread = new Thread();
		midiThread->start((THREAD_FUNCTION)&pollmidi, (void *)this);			
	}
}

void setmidi(const char *str, float channel, float data1, float data2, float dtime)
{
	setsval(lookup("MTYPE", symtab), str);
	setfval(lookup("MCHAN", symtab), channel);
	setfval(lookup("MDATA1", symtab), data1);
	setfval(lookup("MDATA2", symtab), data2);
	setfval(lookup("MDTIME", symtab), dtime);	
}

void *pollmidi(void *m)
{
	MohAwk *mohawk = (MohAwk *)m;
	MidiIn *midi = mohawk->midi;
	MidiMsg msg;
	while(true)
	{
		usleep(10000);
		mohawk->mutex->lock();
		while(midi->recv(&msg))
		{
			int type = msg.data[0] & 0xf0;
			if(midiMessages[type] != NULL)
			{
				setmidi(midiMessages[type], msg.data[0]&0x0f, msg.data[1], msg.data[2], 0);
				mohawk_trigger(winner->narg, midiMessages[type], strlen(midiMessages[type]+1));
				mohawk_trigger(winner->narg, "ONMIDI", 7);
			}
			else
			{
				setmidi("UNKNOWN", msg.data[0]&0x0f, msg.data[1], msg.data[2], 0);
				mohawk_trigger(winner->narg, "ONMIDI", 7);
			}
		}
		mohawk->mutex->unlock();	
	}
	return NULL;
}

void setupMessageTable()
{
	for(int i = 0; i < MIDI_MESSAGES; i++)
	{
		midiMessages[i] = (char *)malloc(16*sizeof(char));
		midiMessages[i][0] = '\0';
	}
  strcpy(midiMessages[128], "NoteOff");
  strcpy(midiMessages[144], "NoteOn");
  strcpy(midiMessages[160], "PolyPressure");
  strcpy(midiMessages[176], "ControlChange");
  strcpy(midiMessages[192], "ProgramChange");
  strcpy(midiMessages[208], "AfterTouch");
  strcpy(midiMessages[224], "PitchBend");
  strcpy(midiMessages[249], "PitchChanged");

  strcpy(midiMessages[248], "Clock");
  strcpy(midiMessages[250], "SongStart");
  strcpy(midiMessages[251], "Continue");
  strcpy(midiMessages[252], "SongStop");
  strcpy(midiMessages[254], "ActiveSensing");
  strcpy(midiMessages[255], "SystemReset");

  strcpy(midiMessages[7], "Volume");
  strcpy(midiMessages[1], "ModWheel");
  strcpy(midiMessages[2], "Breath");
  strcpy(midiMessages[4], "FootControl");
  strcpy(midiMessages[65], "Portamento");
  strcpy(midiMessages[8], "Balance");
  strcpy(midiMessages[10], "Pan");
  strcpy(midiMessages[64], "Sustain");
  strcpy(midiMessages[11], "Expression");

  strcpy(midiMessages[128] , "AfterTouch_Cont");
	return;
}











