/*
 *  MohAwk.h
 *  mosievius
 *
 *  Created by Ari Lazier on Sun Mar 30 2003.
 *  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
 *
 */

#if !defined(__MohAwk_h)
#define __MohAwk_h

#define WSIZE 2048
#define END 0

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <vector>

#include "Thread.h"
#include "Stream.h"
#include "Mixer.h"

#include <map>
#include <vector>
#include <string>


typedef int COLLECTION;
typedef int WINDOW;
typedef int FEATURE;
typedef const string & STREAM;
typedef const string & SOUNDFILE;
typedef int DESCRIPTOR;
typedef float FLOAT;
typedef int STATUS;
typedef int EFFECT;

struct System;

class MidiIn;

class MohAwk
{
	private:
		static MohAwk *instance;
		void midiInit();
		void processInit();
		void extractorsInit();
		Thread *midiThread, *processThread;
		fvec inWin, liveWin, extracted;

	protected:
		MohAwk()
		{
			instance = 0;
			winSize = WSIZE;
			hopSize = winSize/2;
			tickSize = BUF_SIZE;
		}
		~MohAwk();
	
	public:
		static MohAwk *Instance()
		{
			if(instance == 0) instance = new MohAwk();
			return instance;
		}
		void init(const string &home, int wsize = WSIZE, int hsize = WSIZE/2, int tsize = BUF_SIZE);
		Stream *current;
		vector <Stream *> streams;
		Mutex *mutex;
		MidiIn *midi;
		int winSize, hopSize, tickSize;
		bool midiactive, isextracted;
		void tick(int size, fvec &inbuf);
		int symlookup(const char *);

///	State specific functions
/// Plays a window in the current stream.  If no second argument is provided the gain is set to 1.
		STATUS play(WINDOW, FLOAT gain = 1);
/// Sets the current stream to the stream specified.
		STATUS setstream(STREAM);
/// Sets the specified effect for the current stream to value.
		STATUS seteffect(EFFECT, FLOAT value);
		
/// Functions that deal with collections
/// Creates a collection from an audio file.
		COLLECTION openaudio(SOUNDFILE);
/// Extracts features from all windows in a collection
		COLLECTION extract(COLLECTION);	
/// Creates a collection from an audio file and extracts features for each window.
/// Equivalent to calling openaudiofile followed by extract on the returned collection.
		COLLECTION extract(SOUNDFILE);
/// Duplicated the given collection.  If no collection is given, returns and empty collection
		COLLECTION newcollection(COLLECTION collection);
		COLLECTION deletecollection(COLLECTION);
	
		int collectionsize(COLLECTION);
/// Functions which deal with windows
/// Returns the window from the given collection specified by the given descripton.
/// If not descriptor is given, the descriptor NEXT is used.
		WINDOW getwindow(COLLECTION, DESCRIPTOR);
/// Inserts the window in collection before place if it is in collection.
/// If place is not specified the the window is added to the end of the collection.
/// If the place is specfies but is not in the collection then the window is not added.
		STATUS addwindow(COLLECTION, WINDOW, WINDOW position = END);
/// Deletes the specified window from the specified collection if it is in that collection.
		STATUS deletewindow(COLLECTION, WINDOW);

///	Get the specified feature for the specified window.
		FLOAT getfeature(WINDOW, FEATURE);
		FLOAT min(COLLECTION, FEATURE);
		FLOAT max(COLLECTION, FEATURE);
		STATUS sort(COLLECTION, FEATURE);
		void record();
};


#endif






