/*
 *  Stream.cpp
 *  mosievius
 *
 *  Created by Ari Lazier on Sat Apr 12 2003.
 *  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
 *
 */

#include <iostream>
#include "Stream.h"
#include "Mixer.h"
#include "defs.h"
#include "awk.h"
#include "MohAwk.h"

#define WIDTH 16
#define SPERZERO 60
#define USEINTERP true
#define USETABLE false
#define LINEAR true
#define MAX_HOP 44100


Mixer *mixer = Mixer::Instance();
MohAwk *mAwk = MohAwk::Instance();

Stream::Stream(const string str, int winsize, int hopsize, int ticksize)
{
	name = str;
	winSize = winsize;
	hopSize = hopsize;
	tickSize = ticksize;
	num_buffers = 20;
	win.create(num_buffers*hopSize);
	tswin = new fvec [3];
	for(int i = 0; i < 3; i++) tswin[i].create(MAX_HOP);
	cur_buffer = cur_time = 0;
	panWin.create(tickSize);
	outWin.create(tickSize*2);
	pitch = time = srate = 1;
	hopSamples = 2048;
	hop[0] = hop[1] = hop[2] = (int)hopSamples;
	outSamples = 0;
	full_buffers = 0;
	inSample = pitchSample = 0;
	tickable = false;
	angle = 90;
	panRate = 0;
	solaSamples = 0;
}

int
Stream::effects(int effect, float value)
{
	if(effect == mAwk->symlookup("SRATE") && value > .001 && value < 100) srate = value;
	else if(effect == mAwk->symlookup("PSHIFT") && value > .001 && value < 100) pitch = value;
	else if(effect == mAwk->symlookup("TSHIFT") && value > .001 && value < 100) time = value;
	else if(effect == mAwk->symlookup("HOPSZ") && value > 0 && value < 44100) hopSamples = value;
	else if(effect == mAwk->symlookup("PAN") && angle >= 0 && angle <= 360) angle = value;
	else if(effect == mAwk->symlookup("DPAN") && abs(value) < 50) panRate = value;
	else if(effect == mAwk->symlookup("SOLA") && value >= 0 && value < winSize)
		solaSamples = (int)value;
	else return 0;
	return 1;
}

void 
Stream::tick(int size)
{
	if(tickable) mixer->putWin(outWin);
}

void
Stream::addwin(int window, fvec &inWin) /* negative window numbers indicates backwards playback*/
{
//	cout << name << ": time=" << time << " full=" << full_buffers << endl;
	int i, cur = ( cur_buffer + full_buffers ) % num_buffers;
	int curpone = ( cur_buffer + full_buffers + 1 ) % num_buffers;
	if(window > 0) for(i = 0; i < hopSize; i++)
	{
		win(i+cur*hopSize) += inWin(i) * i / (float)hopSize;
		win(i+curpone*hopSize) = inWin(i+hopSize) * (hopSize - i) / (float)hopSize;
	}
	else for(i = 0; i < hopSize; i++)
	{
		win(i+cur*hopSize) += inWin(winSize-i-1) * i / (float)hopSize;
		win(i+curpone*hopSize) = inWin(hopSize-i-1) * (hopSize - i) / (float)hopSize;
	}
	full_buffers++;
	tickable = true;
	return;
}


int
Stream::timeshift(float tim, float pit)
{
	if(!tickable) inSample = 0;
	if((int)inSample >= hopSize)
	{
		int diff = (int)inSample/hopSize, l, m, n;
		inSample -= diff*hopSize;

		for(l = cur_buffer, m = cur_buffer; l < diff + cur_buffer; l++, m++)
		{
			if(m == num_buffers) m = 0;
			for(n = 0; n < hopSize; n++) win(m * hopSize + n) = 0;
		}
		
		full_buffers -= diff;
		cur_buffer = ( cur_buffer + diff ) % num_buffers;
	    if(full_buffers < 0)
		{
			tickable = false;
			full_buffers = 0;
		}
	}
	if(!tickable) mohawk_trigger(winner->narg, name.c_str(), 7);
	if(!tickable) return 0;

	hop[cur_time] = (int)hopSamples;
	int i, j, d, t, max, old_full;
	int offset, size = (int)inSample + hop[0] + hop[1] + hop[2] + solaSamples;
	if(full_buffers * hopSize < size)
	do {
		old_full = full_buffers;
		mohawk_trigger(winner->narg, name.c_str(), 7);
	}
    while(full_buffers * hopSize < size && old_full != full_buffers);
	
	float corr, maxcorr, *windata, *tswindata;
	max = num_buffers * hopSize;
	
	windata = win.getData();
	tswindata = tswin[(cur_time+2)%3].getData();
	for(j = 0, maxcorr = 0, offset = 0; j < solaSamples; j++)
	{
		corr = 0;
		t = (int)inSample + cur_buffer * hopSize + j + hop[(cur_time+1)%3];
		for(i = 0, d = (cur_time + 2) % 3; i < hop[d]; i += 1, t += 1) 
		{
			if(t >= max) t = 0;
			corr += tswindata[i] * windata[t];
		}
		if(corr > maxcorr)
		{
			maxcorr = corr;
			offset = j;
		}
	}
	
	t = (int)inSample + cur_buffer * hopSize + offset;
    for(i = 0, d = (cur_time+1) % 3; i < hop[d]; i++, t++)
	{
		if(t >= max) t = 0;
		tswin[d](i) += i * win(t) / (2*hop[d]);
	}
	for(i = 0, d = (cur_time+2) % 3; i < hop[d]; i++, t++) 
	{
		if(t >= max) t = 0;
		tswin[d](i) = tswin[d](i) * (hop[d]-i) / (2*hop[d]) + .5 * win(t);
	}
	for(i = 0, d = cur_time; i < hop[d]; i++, t++)
	{
		if(t >= max) t = 0;
		tswin[d](i) = win(t);
	}
    inSample += hop[cur_time] / (tim*pit);
	cur_time = (cur_time+1) % 3;
	tswin[cur_time](hop[cur_time]) = tswin[(cur_time+1)%3](0);
	return 1;
}



void
Stream::process()
{
	int i, pintSample, channels = mixer->nChannels();
	float dat, diff, cursrate = srate, curtime = time, curpitch = pitch, temp;
	while(outSamples < tickSize)
	{
		pintSample = (int)pitchSample;
		if(pintSample >= hop[cur_time] || !tickable)
		{
			pitchSample = 0;
			if(!timeshift(curtime, curpitch)) return;
			pintSample = (int)pitchSample;
		}
		diff = pitchSample-pintSample;
		panWin(outSamples++) = tswin[cur_time](pintSample)*(1-diff)+tswin[cur_time](pintSample+1)*diff;
		pitchSample += srate*pitch;
	}
	for(i = 0; i < tickSize; i++, angle += panRate/(float)tickSize)
	{
		while(angle > 360) angle -= 360;
		while(angle < 0) angle += 360;
		temp = angle;
		if(temp < 0) temp = -temp;
		if(temp > 180) temp = 360 - temp;
		outWin(i*2) = panWin(i) * temp / 90.0;
		outWin(i*2+1) = panWin(i) * (2 - temp / 90.0);
	}
	outSamples = 0;
}


//float t_sinc(float x)
//{
	//if(fabs(x)>=WIDTH-1) return 0.0;
	//float temp = fabs(x) * (float) SPERZERO;
	//if(USEINTERP) return linear_interp(sinc_table[(int)temp], sinc_table[(int)temp+1], temp-(int)temp);
	//else return sinc_table[(int)temp];
//}

		//if(LINEAR)
		//else if(cursrate < 1.0) for(i=1, dat=0; i<WIDTH; i++) dat += tswin(i+pintSample)*sinc(diff-i+WIDTH/2);
		//else for(i=1, dat=0; i<WIDTH; i++) dat += tswin(i+pintSample)*sinc((diff-i+WIDTH/2)/cursrate)/cursrate;


float
Stream::sinc(float x)
{
	//if(USETABLE) return t_sinc(x);
	if (x==0.0) return 1.0;
	float temp = PI * x;
	return sin(temp) / (temp);
}



