/*
 *  Accents.cpp
 *  
 *
 *  Created by Ari Lazier on Thu Jun 26 2003.
 *  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
 *
 */

#include "Accents.h"
#define HWR(A)  ( A < 0.0 ? 0.0 : A ) 
#define round(A) ( (A - floor(A)) < 0.5 ? floor(A) : ceil(A) )

Accents::Accents()
{
	outSize_ = 4;
	init(4);
}

Accents::Accents(int insize, int hopsize, int bands)
{
	inSize_ = insize;
	beta = hopsize/2;
	outSize_ = bands;
	init(bands);
}
	
Accents::~Accents() {}

void
Accents::init(int bands)
{
	hamming = new Hamming(inSize_, 0);
	ham_.create(inSize_);
	fft = new MagFFT(inSize_);
	fft_.create(inSize_/2);
        
	nbands = 36;
	nsigs = bands;
	bandborders.create(nbands+2);
	bandpower.create(nbands);
	smoothed.create(nbands);
	oldsmoothed.create(nbands);
	differ.create(nbands);

	// calculate borders for erb bands
	minfreq = 50;
	maxfreq = 20000;

	int i;
	float erbmin = 21.4*log10(.00437 * minfreq + 1);
	float erbmax = 21.4*log10(.00437 * maxfreq + 1);
	float erbwidth = (erbmax - erbmin) / (nbands + 1);
	float erbfreq = erbmin;
	for(i = 0; i < nbands+2; i++, erbfreq += erbwidth) {
		bandborders(i) = round(inSize_ * ( pow(10.0, erbfreq / 21.4) - 1 ) / ( 44100 * .00437 ));
	}
	
	// 6th order lowpass butterworth, 10Hz cutoff coefficients
	butter = new (Filter *)[nbands];
	fvec n(7), d(7);
	n(0)=.0000195; n(1)=.0001171; n(2)=.0002927; n(3)=.0003902; n(4)=.0002927; n(5)=.0001171; n(6)=.0000195;
	d(0)=1; d(1)=-4.5901; d(2)=8.9103; d(3)=-9.3419; d(4)=5.57; d(5)=-1.7885; d(6)=.2414;
	for(i = 0; i < nbands; i++) butter[i] = new Filter(n, d, "lpf1");
	gamma = .1;
	dubbya = 1;
}

void 
Accents::process(fvec &in, fvec& out)
{
	hamming->process(in, ham_);
	fft->process(ham_, fft_);

	int i, j, bwidth, fwidth, swidth, adjecent;
	for(i = 0; i < nbands; i++) {
		// calcualate power for each band
		bandpower(i) = 0;
		fwidth = (int)bandborders(i+1) - (int)bandborders(i);
		swidth = (int)bandborders(i+2) - (int)bandborders(i+1);
		for(j = 0; j < fwidth; j++) bandpower(i) += j * fft_((int)bandborders(i)+j) / fwidth;
		for(bwidth = fwidth+swidth; j < bwidth; j++) bandpower(i) += (bwidth-j) * fft_((int)bandborders(i)+j) / swidth;
		bandpower(i) /= bwidth;

		// take log, filter, hw-rectify the differential, an take the weighted average of smoothed and differ
		bandpower(i) = log(1+gamma*bandpower(i))/log(1+gamma);
		smoothed(i) = butter[i]->tick(bandpower(i));
		differ(i) = HWR(smoothed(i) - oldsmoothed(i));
		differ(i) = (1-dubbya)*smoothed(i) + dubbya*beta*differ(i);
		oldsmoothed(i) = smoothed(i);
	}
        
	// calcluate registral accent signals by summing over adjecent bands
	for(i = 0, adjecent = nbands / nsigs; i < nsigs; i++)
		for(j = 0, out(i) = 0; j < adjecent; j++)
			out(i) += differ(i * adjecent + j);
}







