/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class AiffFileIO
   \brief SoundFileIO for .aiff soundfiles
   
   SoundFileIO reader/writer for .aiff sound files.
   (SGI, MAC aiff audio format). 
*/



#include "AiffFileIO.h"


AiffFileIO::AiffFileIO(string filename, int mode):SoundFileIO(filename, mode)
{
}


AiffFileIO::~AiffFileIO()
{
}

int 
AiffFileIO::check_type()
{
  FormAIFCChunk_Header hdr;
  if (!sfp_)
    {
      cerr << "AiffFileIO::File " << filename_ << " does not exist" << endl;
      return MRS_ERROR;
    }
  if (fread(&hdr, sizeof(FormAIFCChunk_Header), 1, sfp_) == 0)
    {
      cerr << "File " << filename_ << " is empty " << endl;
      return MRS_ERROR;
    }
  
  if ((!strncmp(hdr.formType,"AIFF",4)) && (!strncmp(hdr.chunkID,"FORM",4)))
    {
      return MRS_OK;
    }
  else
    {
      return MRS_ERROR;
    }
  return MRS_OK;
}


void 
AiffFileIO::init()
{
  // Read Header
  Chunk_Header chdr;  
  fread(&chdr, sizeof(chdr), 1, sfp_);  
  if (!strncmp(chdr.chunkID, "ANNO", 4)) // Skip marker chunk for now
    {
      cerr << "READ ANNOTATION CHUNK" << endl;
      chdr.chunkDataSize = ntohl(chdr.chunkDataSize);
      cerr << "DataSize = " << chdr.chunkDataSize << endl;
      fseek(sfp_, chdr.chunkDataSize, 1);
    }
  
  fread(&hdr_, 10, 1, sfp_);
  fread(&(hdr_.length), sizeof(unsigned long), 1, sfp_);
  fread(&(hdr_.sample_size), sizeof(short), 1, sfp_);
  
  //fread(&hdr_, sizeof(CommonChunk_Header), 1, sfp_);
  cerr << "hdr chunkID =" << hdr_.chunkID << endl;
  cerr << "hdr chunkDataSize = " << ntohl(hdr_.chunk_DataSize) << endl;
  cerr << "hdr channels      = " << ntohs(hdr_.channels) << endl;
  cerr << "hdr length = " << ntohl(hdr_.length) << endl;
  cerr << "hdr sample size = " << ntohs(hdr_.sample_size) << endl;
  hdr_.sample_rate = read_ieee_extended();
  cerr << "hdr sample rete = " << hdr_.sample_rate << endl;
  return;
  
  /*hdr.id = 14;
  hdr.length = hdr_.length;
  hdr.sample_rate = (long)hdr_.sample_rate;
  hdr.channels    = hdr_.channels;
  hdr.win_size = DEFAULT_WIN_SIZE;
  hdr.hop_size = DEFAULT_WIN_SIZE;
  
  hdr.duration    = hdr.length / (float)hdr.sample_rate;
  hdr.comments    = "George Tzanetakis, MARSYAS 1998\n";
  */
  /* Read various chunks until SSND chunk */
  //while(1)
  //{
  //fread(&chdr, sizeof(chdr), 1, sfp_);
  /*      cerr << "Chunk ID is " << chdr.chunkID << endl;
	  cerr << "Chunk Data size is " << chdr.chunkDataSize << endl;
  */
  //if (!strncmp(chdr.chunkID, "MARK", 4)) // Skip marker chunk for now
  //{
  // Must add code for markers and use them 
  // with my segmentation stuff
  
  /*	  unsigned short num_of_markers;
	  short mark_id1, mark_id2;
	  unsigned long position1, position2;
	  unsigned char string1_size, string2_size;
	  char str1[256];
	  char str2[256];
	  
	  fread(&num_of_markers, sizeof(short), 1,sfp_);
	  fread(&mark_id1, sizeof(mark_id1), 1, sfp_);
	  fread(&position1, sizeof(position1),1, sfp_);
	  fread(&string1_size, sizeof(string1_size), 1, sfp_);
	  fread(str1, string1_size, 1, sfp_);
	  cerr << "Number of markers is " << num_of_markers << endl;
	  cerr << "Mark id 1 is " << mark_id1 << endl;
	  cerr << "Position 1 is " << position1 << endl;
	  cerr << "String 1 size is " << (int)string1_size << endl;
	  cerr << "String 1 is \"" << str1 << "\"" << endl;

	  fread(&string1_size, sizeof(string1_size), 1, sfp_); 
	  
	  fread(&mark_id2, sizeof(mark_id2), 1, sfp_);
	  fread(&position2, sizeof(position2),1, sfp_);
	  fread(&string2_size, sizeof(string2_size), 1, sfp_);
	  fread(str2, string2_size, 1, sfp_);
	  fread(&string2_size, sizeof(string2_size), 1, sfp_);	  
  */
	  
	  //}
	//  else 
  /*{
	  fseek(sfp_, chdr.chunkDataSize, 1);
	}
    }
  */
  /*dest.init_header(hdr); */
  //return MRS_OK;
}


void
AiffFileIO::get(fvec &win, unsigned long index)
{
}


void 
AiffFileIO::put(fvec &win, unsigned long index)
{
}

  
  



	
