/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/** 
    \class DB
    \brief Convert to DBs

    Convert values of input fvec to DBs. (20log10()).
*/




#include "DB.h"



DB::DB()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = DEFAULT_WIN_SIZE;
}


DB::DB(unsigned int inSize, float floor)
{
  inSize_ = inSize;
  outSize_ = inSize;
  floor_  = floor;
}

void
DB::process(fvec& in, fvec& out)
{
  unsigned int i;
  if ((in.size() != inSize_) || ((out.size() != outSize_)))
    {
      cerr << "Warning: DB::process: inSize_ and input window size do not agree" << endl;
    }

  
  for (i=0; i<inSize_; i++)
    {
      
      if (in(i) < floor_) out(i) = 20.0 * log10(floor_);
      else
	out(i) = 20.0 * log10(in(i));
      
      
    }
}







