/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class FeatMatrix 
    \brief Matrix of feature values 

A matrix of features values (fmatrix) typically calculated with an Extractor. In
addition to the feature values it includes information about 
the feature names and class names. 
*/




#include "FeatMatrix.h"

FeatMatrix::FeatMatrix():fmatrix()
{
  name_ = "DefaultFeatMatrix";
  labelNames_.reserve(100);
  featNames_.reserve(100);
}

FeatMatrix::FeatMatrix(unsigned long rows):fmatrix(rows)
{
  name_ = "DefaultFeatMatrix";
  labelNames_.reserve(100);
  featNames_.reserve(100);
  labels_.create(rows,1);
}



FeatMatrix::~FeatMatrix()

{
}


FeatMatrix::FeatMatrix(unsigned long rows, unsigned long cols):fmatrix(rows, cols)
{
  name_ = "DefaultFeatMatrix";
  labelNames_.reserve(100);
  featNames_.reserve(100);
  labels_.create(rows,1);
}



FeatMatrix::FeatMatrix(FeatMatrix& full, ivec& partition, int flag)
{
  rows_ = 0;
  unsigned int frows;
  unsigned int r,c,l;
  
  frows = partition.size();
  
  for (r=0; r < frows; r++)
    {
      if (partition(r) == flag) 
	rows_++;
    }
  cols_ = full.cols();
  name_ = full.name() + "Train";
  labelNames_ = full.labelNames_;
  featNames_ = full.featNames_;
  labels_.create(rows_, full.labels_.cols());  

  data_ = new float[rows_ * cols_];
  size_ = rows_ * cols_;
  
  unsigned int rn = 0;


  unsigned int lcols;
  lcols = labels_.cols();
  
  for (r=0; r < frows; r++)
    {
      if (partition(r) == flag) 
	{
	  for (c=0; c < cols_; c++)
	    data_[rn * cols_ + c] = full(r,c);
	  for (l=0; l < lcols; l++)
	    labels_(rn,l) = full.labelNum(r,l);
	  rn++;
	}
    }
  
  labelSizes_.create(lcols);
  for (l=0; l < lcols; l++)
    labelSizes_(l) = full.labelSize(l);

  row_.create(cols_);
  col_.create(rows_);
  meanRow_.create(cols_);
  stdRow_.create(cols_);
  varRow_.create(cols_);  
}




FeatMatrix::FeatMatrix(vector <FeatMatrix *> matrices)
{
  cout << "deprecated" << endl;
  /* 
  unsigned int r,c;
  unsigned int i;
  name_ = "FromCollection";
  featSize_ = matrices[0]->featSize();
  classSize_ = matrices[0]->classSize();
  rows_ = matrices[0]->rows();
  for (i=1; i< matrices.size(); i++)
    {
      if (matrices[i]->featSize() != featSize_)
	{
	  cerr << "FeatMatrix::FeatMatrix(vector <FeatMatrix *>) maps have different featSize " << endl;
	  cerr << "featSize_ = " << featSize_;
	  cerr << "matrices = " << matrices[i]->featSize() << endl;
	  cerr << "i = " << i << endl;
	  cerr << "size = " << matrices.size() << endl;
	  return;
	}
      if (matrices[i]->classSize() != classSize_)
	{
	  cerr << "FeatMatrix::FeatMatrix(vector <FeatMatrix *>) maps have different classSize " << endl;
	  return;
	}
      rows_ += matrices[i]->rows();
    }
  classNames_ = matrices[0]->classNames();
  featNames_ = matrices[0]->featNames();
  cols_ = featSize_;
  
  
  row_.create(cols_);
  col_.create(rows_);  
  meanRow_.create(cols_);
  varRow_.create(cols_);
  stdRow_.create(cols_);  
  data_ = new float[rows_ * cols_];
  size_ = rows_ * cols_;
  
  classVec_.create(rows_);
  unsigned int mrows, mcols, msize;
  unsigned int mstart =0;
  unsigned int mrowstart = 0;
  
  for (i=0; i< matrices.size(); i++)
    {
      mrows = matrices[i]->rows();
      mcols = matrices[i]->cols();
      msize = mrows * mcols;
      for (r = 0; r < mrows; r++)
	{
	  classVec_(mrowstart + r) = matrices[i]->classVec_(r);
	  for (c = 0; c < mcols; c++)
	    {
	      data_[mstart + r * cols_ + c] = matrices[i]->operator()(r,c);
	    }
	}
      mstart += msize;
      mrowstart += mrows;
    }
  */

}





FeatMatrix::FeatMatrix(FeatMatrix **matrices, unsigned int numMatrices)
{
  cout << "deprecated" << endl;
  /* 
  unsigned int r,c;
  unsigned int i;
  name_ = "FromCollection";
  featSize_ = matrices[0]->featSize();
  classSize_ = matrices[0]->classSize();
  rows_ = matrices[0]->rows();
  for (i=1; i< numMatrices; i++)
    {
      if (matrices[i]->featSize() != featSize_)
	{
	  cerr << "FeatMatrix::FeatMatrix(vector <FeatMatrix *>) maps have different featSize " << endl;
	  return;
	}
      if (matrices[i]->classSize() != classSize_)
	{
	  cerr << "FeatMatrix::FeatMatrix(vector <FeatMatrix *>) maps have different classSize " << endl;
	  return;
	}
      rows_ += matrices[i]->rows();
    }
  

  classNames_ = matrices[0]->classNames();
  featNames_ = matrices[0]->featNames();
  cols_ = featSize_;

  row_.create(cols_);
  col_.create(rows_);  
  meanRow_.create(cols_);
  varRow_.create(cols_);
  stdRow_.create(cols_);  
  data_ = new float[rows_ * cols_];
  size_ = rows_ * cols_;


  unsigned int mrows, mcols, msize;
  unsigned int mstart =0;
  unsigned int mrowstart = 0;

  for (i=0; i< numMatrices; i++)
    {
      mrows = matrices[i]->rows();
      mcols = matrices[i]->cols();
      msize = mrows * mcols;
      for (r = 0; r < mrows; r++)
	{
	  classVec_(mrowstart + r) = matrices[i]->classVec_(r);
	  for (c = 0; c < mcols; c++)
	    {
	      data_[mstart + r * cols_ + c] = matrices[i]->operator()(r,c);
	    }
	}
      mstart += msize;
      mrowstart += mrows;
    }
  */
}


void 
FeatMatrix::paste(int offset, FeatMatrix* fmatrix)
{
  cout << "deprecated" << endl;
  /* 
  cout << "FeatMatrix called" << endl;
  cout << "offset = " << offset << endl;
  
  int r,c;
  int frows = fmatrix->rows();
  int fcols = fmatrix->cols();
  cout << "Frows, fcols " << frows << " " << fcols << endl;
  cout << "rows_ = " << rows_ << endl;
  cout << "cols_ = " << cols_ << endl;
  cout << "size_ = " << size_ << endl;
  
  
  for (r=offset; r < offset+frows; r++)
    for (c=0; c < fcols; c++)
      {
	if (r >rows_) cout << "PROBLEM" << endl;
	if (c >cols_) cout << "CPROBLEM" << endl;
	if (r * cols_ + c > size_) cout << "SPROBLEM" << endl;
	data_[r * cols_ + c] = 5.0;
      }
  
  return;
  // fmatrix->operator()(r-offset,c);
  */
}



void 
FeatMatrix::concat(FeatMatrix& fmatrix1, FeatMatrix& fmatrix2)
{
  cout << "deprecated" << endl;
  /* 
  unsigned int r,c;
  unsigned int i;
  unsigned int debug_rows;
  name_ = "FromConcat";
  featSize_ = fmatrix1.featSize() + fmatrix2.featSize();
  classSize_ = fmatrix2.classSize();
  rows_ = fmatrix1.rows();
  cols_ = fmatrix1.cols() + fmatrix2.cols();
  
  classNames_ = fmatrix2.classNames();
  
  vector<string> fnames1;
  vector<string> fnames2;
  fnames1 = fmatrix1.featNames();
  fnames2 = fmatrix2.featNames();

  for (i=0; i < fmatrix1.featSize(); i++)
    {
      featNames_.push_back(fnames1[i]);
    }


  for (i=0; i < fmatrix2.featSize(); i++)
    {
      featNames_.push_back(fnames2[i]);
    }
  
  // ?? featNames_ = matrices[0]->featNames();
  
  row_.create(cols_);
  col_.create(rows_);
  meanRow_.create(cols_);
  varRow_.create(cols_);
  stdRow_.create(cols_);  
  size_ = rows_ * cols_;
  classVec_.create(rows_);
  
  
  for (r = 0; r < rows_; r++)
  {
    classVec_(r) = fmatrix2.classVec_(r);    
      for (c=0; c < cols_; c++) 
	{
	  if (c < fmatrix1.cols())
	    data_[r * cols_ + c] = fmatrix1(r,c);
	  else 
	    data_[r * cols_ + c] = fmatrix2(r, c - fmatrix1.cols());
	}
    }
 
  */
  
}





/* 
void 
FeatMatrix::label(unsigned int start, unsigned int end, unsigned int label)
{
  classVec_.setval(start, end, label);  
}


void 
FeatMatrix::label(vector<string> labels, unsigned int label)
{
  classNames_ = labels;
  classSize_ = labels.size();
  classVec_.setval(label);
}
*/

/* 
void
FeatMatrix::changefeatNames(vector<string> featNames)
{
  unsigned int i;
  if (featNames_.size() != featNames.size()) 
    {
      cerr << "FeatMatrix::changefeatNames: featNames different sizes" << endl;
      return;
    }  
  for (i=0; i < featSize_; i++)
    featNames_[i] = featNames[i];
}
*/

 
void 
FeatMatrix::setLabelNames(vector<string> labelNames)
{
  unsigned int cl;
  if (labelNames_.size() != 0)
    {
      cerr << "FeatMatrix::setlabelNames: labelNames already allocated\n" << endl;
      return;
    }
  for (cl=0; cl < labelNames.size(); cl++)
    {
      labelNames_.push_back(labelNames[cl]);
    }
}


 
void
FeatMatrix::setFeatNames(vector<string> featNames)
{
  unsigned int i;
  if (featNames_.size() != 0) 
    {
      cerr << "FeatMatrix::featNames: featNames already allocated\n" << endl;
      return;
    }
  unsigned int featSize = featNames.size();
  for (i=0; i< featSize; i++)
    {
      featNames_.push_back(featNames[i]);
    }
}


/* 
void 
FeatMatrix::setClassVec(ivec& classVec)
{
  classVec_ = classVec;
}
*/


string 
FeatMatrix::name()
{
  return name_;
}


void 
FeatMatrix::setName(string name)
{
  name_ = name;
}


vector<string>
FeatMatrix::featNames()
{
  return featNames_;
}

vector<string>
FeatMatrix::labelNames()
{
  return labelNames_;
}

/* 
string
FeatMatrix::className(unsigned int classNum)
{
  return classNames_[classNum];
}
*/

void 
FeatMatrix::setLabelRange(unsigned int start, unsigned int end, unsigned int l,
			  unsigned int value)
{
  unsigned int r;
  for (r =start; r < end; r++)
    {
      labels_(r,l) = value;
    }
}


void 
FeatMatrix::setLabel(unsigned int r, unsigned l, unsigned int value)
{
  labels_(r,l) = value;
}


unsigned int 
FeatMatrix::labelNum(unsigned int r, unsigned l)
{
  return labels_(r, l);
}

/* 
void 
FeatMatrix::mask(ivec mask)
{
  if (cols_ != mask.size())
    {
      cerr << "Mask has wrong size. All features will be used " << endl;
      cerr << "Feat size = " << cols_ << endl;
      cerr << "Mask size = " << mask.size() << endl;
      return;
    }
  unsigned int r, c;
  
  
  for (r=0; r < rows_; r++)
    {
      for (c=0; c< cols_; c++)
	{
	  if (mask(c) == 0)
	    data_[r * cols_ + c] = 0.0;
	}
    }
}
*/

unsigned int 
FeatMatrix::labelCount(unsigned int l, unsigned int cl)
{
  unsigned int r;
  unsigned int count;
  
  count =0;
  for (r=0; r < rows_; r++)
    {
      if (labels_(r,l) == cl)
	count ++;
    }
  return count;
}



unsigned int 
FeatMatrix::labelSize(unsigned int label)
{
  return labelSizes_(label);
}

unsigned int 
FeatMatrix::featSize()
{
  return featNames_.size();
}



void
FeatMatrix::classStat(unsigned int l, fmatrix& classMeans, fmatrix& classVars)
{
  unsigned int c = 0;
  unsigned int r = 0;
  unsigned int cl = 0;
  unsigned int classSize = labelSizes_(l);
  
  classMeans.setval(0.0);
  classVars.setval(0.0);
  fvec classSizes_(classSize);
  classSizes_.setval(0.0);
  float val;
  for (r=0; r < rows_; r++)
    {
      for (c=0; c< cols_; c++)
	{
	  cl = labels_(r,l);
	  val = data_[r * cols_ + c];
	  classMeans(cl, c) += val;
	  classVars(cl, c) += (val * val);
	}
      classSizes_(cl)++;
    }
  
  for (cl =0; cl < classSize; cl++)
    for (c =0; c < cols_; c++)
      {
	if (classSizes_(cl) != 0) 
	  {
	    classMeans(cl,c) /= classSizes_(cl);
	    classVars(cl,c) /= classSizes_(cl);
	  }
      }
  for (cl = 0; cl < classSize; cl++)
    for (c=0; c < cols_; c++)
      {
	val = classMeans(cl, c);
	classVars(cl, c) -= (val * val); 
      }
}




void 
FeatMatrix::read(string name)
{
  unsigned int i;
  string str;

  // Read features
  string fstring = name + ".mff";
  ifstream fis(fstring.c_str());
  for (i=0; i < 3; i++)
    fis >> str;
  fis >> (*this);
  
  // Read labels
  fstring = name + ".mlf";
  ifstream lis(fstring.c_str());
  for (i=0; i < 3; i++)
    lis >> str;
  lis >> labels_;

  // Read description 
  fstring = name + ".mdf";
  ifstream dis(fstring.c_str());
  for (i=0; i < 6; i++)
    dis >> str;
  
  unsigned int featSize;
  dis >> featSize;
  if (featNames_.size() != 0) 
    cout << "featNames_ already allocated\n" << endl;
  for (i=0; i < featSize; i++)
    {
      dis >> str;
      featNames_.push_back(str);
    }

  for (i=0; i < 3; i++)
    dis >> str;

  unsigned int labelSize;
  dis >> labelSize;
  if (labelNames_.size() != 0) 
    cout << "labelNames_ already allocated\n" << endl;
  for (i=0; i < labelSize; i++)
    {
      dis >> str;
      labelNames_.push_back(str);
    }
  
  

  unsigned int lcols;
  lcols = labels_.cols();
  
  labelSizes_.create(lcols);
  ivec plabels(lcols);
  plabels.setval(1);
  
  unsigned int r,l;
  
  for (r=0; r < rows_; r++)
    for (l=0; l < lcols; l++)
      {
	if (labels_(r,l) != plabels(l)) 
	  { 
	    labelSizes_(l) = labelSizes_(l) + 1;
	  }
	plabels(l) = labels_(r,l);
      }
}

void
FeatMatrix::write(string name)
{
  unsigned int i;
  
  // Write description 
  string dstring = name + ".mdf";
  ofstream dos(dstring.c_str());
  dos << "# MARSYAS Description\n";
    unsigned int featSize;
  featSize = featNames_.size();
  
  unsigned int labelSize;
  labelSize = labelNames_.size();
  
  dos << "# Feature names " << endl;
  dos << featSize << endl << endl;
  
  for (i=0; i < featSize; i++)
    dos << featNames_[i] << endl;
  dos << endl;
  
  dos << "# Label names " << endl;
  dos << labelSize << endl;
  
  for (i=0; i < labelSize; i++)
    dos << labelNames_[i] << endl;

  
  // Write features
  string fstring = name + ".mff";
  ofstream fos(fstring.c_str());

  fos << "# MARSYAS Features" << endl;
  fos << (fmatrix &)(*this);	// output parent fmatrix

  // Write lables
  fstring = name + ".mlf";
  ofstream los(fstring.c_str());
  los << "# MARSYAS Labels" << endl;
  los << labels_ << endl;
}

void 
FeatMatrix::printDescription(ostream& o) 
{
  unsigned int featSize;
  featSize = featNames_.size();
  o << "# MARSYAS Description" << endl;
  o << "# name = " << name_ << endl;
  o << endl;
  o << featSize << endl;
  unsigned int i;
  
  for (i=0; i < featSize; i++)
    o << featNames_[i];
}



void 
FeatMatrix::printLabels(ostream& o) 
{
  o << labels_;
}



ostream&
operator<<(ostream& o, const FeatMatrix& fm)
{
  o << (fmatrix &)fm;			// output parent fmatrix
  o << endl << endl;
  return o;
}





  


