/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/** 
    \class FeatMatrix 
    \brief Matrix of feature values 

A matrix of features values (fmatrix) typically calculated with an Extractor. In
addition to the feature values it includes information about 
the feature names and class names. 
*/




#if !defined(__FeatMatrix_h)
#define __FeatMatrix_h

#include "ivec.h"
#include "fmatrix.h"
#include "imatrix.h"
#include <vector>


class FeatMatrix: public fmatrix
{
private: 
  imatrix labels_;
  ivec labelSizes_;
  
  vector<string> labelNames_;
  vector<string> featNames_;
  
  
public:

  FeatMatrix();
  ~FeatMatrix();
  FeatMatrix(unsigned long rows);
  FeatMatrix(unsigned long rows, unsigned long cols);
  FeatMatrix(FeatMatrix& full, ivec& partition, int flag);
  FeatMatrix(vector <FeatMatrix *> matrices);
  FeatMatrix(FeatMatrix** matrices, unsigned int numMatrices);

  void printLabels(ostream& o);
  void printDescription(ostream& o);
  

  void paste(int offset, FeatMatrix* fmatrix);
  void concat(FeatMatrix& fmatrix1, FeatMatrix& fmatrix2);
  void changefeatNames(vector<string> featNames);
  void setFeatNames(vector<string> featNames);
  void setLabelNames(vector<string> classNames);
  void setClassVec(ivec& classVec);
  void setName(string name);
  string name();
  void mask(ivec mask);
  // friend ostream& operator<<(ostream&, const FeatMatrix&);
  // friend istream& operator>>(istream&, FeatMatrix&);
  void read(string filename);
  void write(string filename);

  
  void label(vector<string> labels, unsigned int label);

  void setLabelRange(unsigned int start, unsigned int end, unsigned int l, unsigned int value);
  void setLabel(unsigned int r, unsigned int l, unsigned int value);
  unsigned int labelSize(unsigned int label);
  unsigned int featSize();
  unsigned int labelCount(unsigned int l, unsigned int cl);
  vector<string> featNames();
  vector<string> labelNames();
  string className(unsigned int classNum);
  void classStat(unsigned int l,fmatrix& classMeans, fmatrix& classVars);
  unsigned int labelNum(unsigned int row, unsigned int l);
};


#endif







	
