/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class Parallel
    \brief Parallel connection of System objects
    
    Combines a series of System objects to a single System 
corresponding to executing the System objects in Parallel 
on the same input fvec and concatenating their output fvec objects.
*/


#if !defined(__Parallel_h)
#define __Parallel_h

#include "System.h"

class Parallel: public System
{
private:
  unsigned int numFeats_;
  vector <System*> featArray_;		
  fvec* vectors_;
  
public:
  Parallel();
  Parallel(vector<System*> featArray);
  ~Parallel();
  void process(fvec& in, fvec& out);
};


#endif


