#include "Pitch.h"
#define MIN(a,b) ((a<=b)?(a):(b))
#define MAX(a,b) ((a>=b)?(a):(b))
#define INFINITE 1000000

Pitch::Pitch(int inSize)
{
  inSize_ = inSize;
	outSize_ = 2;
  init();
}


Pitch::~Pitch()
{
}


void
Pitch::init()
{
	nBins_ = inSize_/2;
	resolution_ = 44100.0/inSize_;
	spectrum_.create(inSize_/2);
	
	peaks_.create(nBins_);
	amps_.create(nBins_);
	phases_.create(nBins_);
	partials_.create(nBins_);
	candidates_.create(nBins_);
	error_.create(nBins_);
	
	spectralPeaks_ = new SpectralPeak(inSize_);
}


void
Pitch::process(fvec& in, fvec& out)
{	
	int i, j, k, maxIndex, npartials, goodPeaks, nCandidates = 0;
	float maxAmp, maxAmps[3], a=-resolution_/100.0, b, interval, tmp, lowestBin = 85/resolution_;
	float highestPartialBin = 8000/resolution_, highestFreqBin = 4000/resolution_;
	out(0) = 0;
	out(1) = 10000;
	spectralPeaks_->process(in, peaks_);
	npartials = spectralPeaks_->numberOfPartials();
	if(npartials < 2) return;
  amps_ = spectralPeaks_->getAmps();
	phases_ = spectralPeaks_->getPhases();
	spectrum_ = spectralPeaks_->getSpectrum();
	
	// eliminate bad peaks	
	for(i = 0, maxIndex = 0; i < npartials; i++) if(amps_(i) > amps_(maxIndex)) maxIndex = i;
  maxAmp = amps_(maxIndex);
	for(i = 0; peaks_(i) < lowestBin && i < npartials;) i++;
  for(goodPeaks = 0; i < maxIndex && peaks_(i) <= highestPartialBin; i++)
	  if(amps_(i) >= maxAmp - 30)
	  {
		  peaks_(goodPeaks) = peaks_(i);
			amps_(goodPeaks) = amps_(i);
			goodPeaks++;
		}
	for(b=maxAmp-50-a*peaks_(maxIndex); i < npartials && peaks_(i) <= highestPartialBin; i++)
	  if(amps_(i) >= (a * peaks_(i) + b))
		{
		  peaks_(goodPeaks) = peaks_(i);
			amps_(goodPeaks) = amps_(i);
			goodPeaks++;		  
		}		
	if(goodPeaks == 0) return;
	for(i = 0, maxIndex = 0; i < goodPeaks; i++) if(amps_(i) > amps_(maxIndex)) maxIndex = i;
  maxAmp = amps_(maxIndex);
	
	//for(i = 0; i < goodPeaks; i++) cout<<peaks_(i) << " " << amps_(i) << endl;
	
	// choose candiates
  for(i = 0; i < maxIndex; i++)  								// partials below the highest in amplitude
	  if(peaks_(i) <= highestFreqBin) candidates_(nCandidates++) = peaks_(i);
	for(i = 0; i < goodPeaks; i++)								// intervals between partials
		for(j = i+1; j < goodPeaks; j++)
		{
			interval = peaks_(j) - peaks_(i);
			if(interval < peaks_(maxIndex)*1.1 && interval <= highestFreqBin && interval >= lowestBin)
			{
			  k = 0;
			  while(k<nCandidates && (interval<=candidates_(k)*.9524 || interval>=candidates_(k)*1.05)) k++;
				if(k == nCandidates) candidates_(nCandidates++) = interval;
			}
		}
	for(i = 0; i < goodPeaks; i++)								// submultiples of partials
	  for(j = 1; j <= 5; j++)
		{
			interval = peaks_(i)/j;
			if(interval < maxIndex * 1.1 && interval <= highestFreqBin && interval >= lowestBin)
			{
			  k = 0;
			  while(k<nCandidates && (interval<=candidates_(k)*.95 || interval>=candidates_(k)*1.1)) k++;
				if(k == nCandidates) candidates_(nCandidates++) = interval;
			}
	  }
  if(nCandidates == 0) return;
 
	// calculate error and sort candidates by lowest error
	for(i = 0; i < nCandidates; i++)
	{
	  error_(i) = getError(candidates_(i), maxAmp, goodPeaks);
	  for(j = i; j > 0 && error_(j) < error_(j-1); j--)
	  {
	    tmp = error_(j-1);
			error_(j-1) = error_(j);
			error_(j) = tmp;
			tmp = candidates_(j-1);
			candidates_(j-1) = candidates_(j);
			candidates_(j) = tmp;
		}
	}
	
	//ofstream cand("cand"), er("er");
		
	int num = MIN(3, nCandidates);
	out(0) = candidates_(0);
	out(1) = error_(0);
	for(i = 0; i < num; i++)
	{
	  float freq, error, min = candidates_(i) * .9070, max = candidates_(i) * 1.1025;
		for(freq = min; freq < max; freq += .025*candidates_(i))
		{
		  if((error = getError(freq, maxAmp, goodPeaks)) <= out(1))
			{
				out(0) = freq;
				out(1) = error;
			}
		//cand << freq << endl;
		//er << error << endl;
		}
	}
	return;
}



float	
Pitch::getError(float bin, float maxAmp, int nPeaks)
{
	int i, peakIndex = 0, numPM, numMP;
	float errorPM = 0, errorMP = 0, highestBin = peaks_(nPeaks-1)+bin/2.0;
	float binoffset, peak, amp, tmp, ampoffset, harmonic = bin, freq;
	
	// predicted to measured mismatch error
	for(numPM = 0; harmonic <= highestBin; numPM++)
	{
		peakIndex = getClosest(harmonic, peakIndex, nPeaks);
		binoffset = fabsf(peaks_(peakIndex)-harmonic)*resolution_ * 
					pow((double)harmonic*resolution_, -0.5);
		ampoffset = pow(10.0, (amps_(peakIndex)-maxAmp)/20.0);
		errorPM += binoffset + ampoffset * (1.4 * binoffset - .5);
		harmonic += bin;
	}
 
	// measured to predicted mismatch error 
	for(numMP = 0; numMP < nPeaks; numMP++)
	{
	  harmonic = MAX(1, round(peaks_(numMP)/bin)) * bin;
		binoffset = fabsf(harmonic-peaks_(numMP))*resolution_*pow((double)peaks_(numMP)*resolution_, - .5);
		ampoffset = pow(10.0, (amps_(numMP)-maxAmp)/20.0);
		errorMP += binoffset + ampoffset * (1.4 * binoffset - .5);
	}
		
	//cout << bin << " " << errorPM << " " << errorMP << endl;
	// total error
	return (errorPM / numPM + .33 * errorMP / numMP);
}


int
Pitch::getClosest(float bin, int peakIndex, int nPeaks)
{
  int bestpeak = peakIndex;
  float distance = INFINITE, nextdistance = 0;
  while (peakIndex < nPeaks)
	{
		nextdistance = fabsf(bin - peaks_(peakIndex));
		if(nextdistance >= distance) return bestpeak;
		else
		{
	    bestpeak = peakIndex; 
	    distance = nextdistance;
	    peakIndex++;
		}
	}
  return bestpeak;
}



fvec& Pitch::getPeaks()
{
  return peaks_;
}


fvec& Pitch::getAmps()
{
  return amps_;
}


fvec& Pitch::getSpectrum()
{
  return spectrum_;
}


fvec& Pitch::getPhases()
{
  return phases_;
}

int Pitch::numberOfPartials()
{
  return spectralPeaks_->numberOfPartials();
}


inline int
Pitch::round(float r)
{
  if(r - floor(r) <= 0.5 ) return (int)floor(r);
  return (int)ceil(r);
}







