/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class ShapeDesc
   \brief Shape Descriptos of input fvec

   Descriptors of shape of the input fvec window. 
This system is an example of a parallel connection 
of System objects more specifically Centroid, Rolloff, Flux.
*/


#include "ShapeDesc.h"

ShapeDesc::ShapeDesc()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = 3;
  init();
}

ShapeDesc::ShapeDesc(unsigned int inSize)
{
  inSize_ = inSize;
  outSize_ = 3;
  init();
}

ShapeDesc::~ShapeDesc()
{
  delete centroid_;
  delete rolloff_;
  delete flux_;
  delete parallel_;
}


void 
ShapeDesc::init()
{
  centroid_ = new Centroid(inSize_);
  rolloff_ = new Rolloff(inSize_, 0.80);
  flux_ = new Flux(inSize_);

  vector<System *> featArray;
  featArray.push_back(centroid_);
  featArray.push_back(rolloff_);
  featArray.push_back(flux_);
  parallel_ = new Parallel(featArray);
}



void 
ShapeDesc::process(fvec& in, fvec& out) 
{
  if ((in.size() != inSize_) || (out.size() != outSize_))
    {
      cerr << "Warnging: ShapeDesc::process:  inSize_ and input window size do not agree" << endl;
      return;
    }
  parallel_->process(in, out);
}


	

	
