/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class Spectral1
   \brief FFT based Spectral1 features 


   Calculate FFT mangitude based features. 
The Centroid, Rolloff and Flux are calculated 
on the magnitude spectrum of the input fvec 
and Zcrs are calculated in time domain. 
This System feature are typically used in classification. 
*/

#if !defined(__Spectral1_h)
#define __Spectral1_h

#include "System.h"	
#include "DB.h"
#include "MagFFT.h"
#include "Hamming.h"
#include "Series.h"
#include "ShapeDesc.h"
#include "Zcrs.h"
#include "RMS.h"

class Spectral1: public System
{
private:
  unsigned int zeroSize_;
  Series* series_;
  Hamming* hamming_;
  MagFFT* magfft_;
  ShapeDesc* shapeDesc_;
  Zcrs* zcrs_;
  RMS* rms_;
  fvec temp0_;
  fvec temp1_;
  fvec temp2_;
  fvec temp3_;
  fvec temp4_;
  fvec temp5_;
  fvec temp6_;
  
  
public:
  Spectral1();
  Spectral1(unsigned int inSize, unsigned int zeroSize);
  ~Spectral1();
  void init();
  void process(fvec& in, fvec& out);
};

#endif

	
