/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class Util
    \brief Various utilities

    Util is a class bundling together various rather 
unrelated convenience utilities. 
*/


#include "Util.h"

Util::Util()
{
}


Util::~Util()
{
}


SoundFileIO *
Util::sfopen(string filename, int mode)
{
  int unknown = 1;
  SoundFileIO *frm;
  frm = NULL;
  
  if (mode == MRS_SF_READ)
    {
      SoundFileIO *formats[4];
      AuFileIO *au_format = new AuFileIO(filename, mode);
      AiffFileIO *aiff_format = new AiffFileIO(filename, mode);
      WavFileIO *wav_format = new WavFileIO(filename, mode);
      Mp3FileIO *mp3_format = new Mp3FileIO(filename, mode);
      
      formats[0] = au_format;
      formats[1] = aiff_format;
      formats[2] = wav_format;
      formats[3] = mp3_format;
      
      
      for (int i=0; i<4; i++)
	{
	  if (formats[i]->check_type() == MRS_OK)
	    {
	      formats[i]->init();
	      unknown = 0;
	      frm = formats[i];
	    }
	  else
	    {
	      delete formats[i];
	    }
	}
    }
  return frm;
}



	
      
  
